from setuptools import setup

setup(
    name='life4safe-commons',
    version='0.0.2',
    author='TIT CS',
    author_email='contato@titcs.com.br',
    packages=['live4safe_commons'],
    description='Common functions to liveliness algorithms',
    long_description='Common functions to liveliness algorithms',
    url='https://TITBrasil@dev.azure.com/TITBrasil/TITCS%20-%20Live4Safe/_git/Live4Safe_Common',
    project_urls={
        'Código fonte': 'https://TITBrasil@dev.azure.com/TITBrasil/TITCS%20-%20Live4Safe/_git/Live4Safe_Common',
        'Download': 'https://dev.azure.com/TITBrasil/TITCS%20-%20Live4Safe/_git/Live4Safe_Common'
    },
    license='MIT',
    keywords='live4safe',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: Portuguese (Brazilian)',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Internationalization',
        'Topic :: Scientific/Engineering :: Physics'
    ]
)
